/**
 * Copyright 2017 Confluent Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * Reads binary protobuf messages from stdin, parses and prints them on stdout.
 * The binary message must be preceeded by a 4-byte word containing the length
 * of the binary message, in network byte order (kafkacat -f '%R%s').
 */

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <arpa/inet.h>

#include "print-json.h"

#define MAGIC_BYTE_PROTOBUF 127

static void reader (FILE *fp) {
        const size_t bufsz = 16*1024;
        char *buf;

        setbuf(fp, NULL);

        buf = malloc(bufsz);

        while (1) {
                Monitoring__MonitoringMessage *m;
                int32_t length;

                if (fread(&length, sizeof(length), 1, fp) != 1)
                        break; /* EOF */

                length = ntohl(length);
                if (length > (int32_t)bufsz) {
                        fprintf(stderr, "%% Message too large (%"PRId32")\n",
                                length);
                        exit(1);
                }

                if (fread(buf, length, 1, fp) != 1)
                        break; /* EOF */

                if (buf[0] != MAGIC_BYTE_PROTOBUF) {
                        fprintf(stderr,
                                "%% Failed to parse message "
                                "(invalid MagicByte 0x%x): skipping\n",
                                (int)buf[0]);
                        continue;
                }

                m = monitoring__monitoring_message__unpack(NULL, length-1,
                                                           (uint8_t *)buf+1);
                if (!m) {
                        fprintf(stderr,
                                "%% Failed to parse message of "
                                "length %"PRId32": skipping\n", length-1);
                        continue;
                }

                monitoring_message_print_json(m);

                monitoring__monitoring_message__free_unpacked(m, NULL);
        }

        free(buf);

}

int main (int argc, char **argv) {
        int i;

        for (i = 1 ; i < argc ; i++) {
                fprintf(stderr,
                        "Usage: %s\n"
                        "\n"
                        "Reads length-prefixed (int32 network order) "
                        "binary protobuf messages from stdin, parses "
                        "and prints on stdout.\n"
                        "\n"
                        "Example usage:\n"
                        " kafkacat -b <broker> -C -t _confluent-monitoring -f %%R%%s | %s\n"
                        "\n",
                        argv[0], argv[0]);
                exit(1);
        }

        reader(stdin);
        return 0;
}
